@@ -39,7 +39,7 @@ copyright = "2020, ComPWA"  # noqa: A001
 author = "Common Partial Wave Analysis"
 
 # https://docs.readthedocs.io/en/stable/builds.html
-BRANCH = os.environ.get("READTHEDOCS_VERSION", default="stable")
+BRANCH = os.environ.get("READTHEDOCS_VERSION", "stable")
 if BRANCH == "latest":
     BRANCH = "main"
 if re.match(r"^\d+$", BRANCH):  # PR preview
@@ -198,11 +198,20 @@ nitpicky = True  # warn if cross-references are missing
 nitpick_ignore = [
     ("py:class", "ipywidgets.widgets.widget_float.FloatSlider"),
     ("py:class", "ipywidgets.widgets.widget_int.IntSlider"),
+    (
+        "py:class",
+        "sympy.tensor.array.expressions.array_expressions.ArraySymbol",
+    ),
     ("py:class", "typing_extensions.Protocol"),
 ]
 
 
 # Intersphinx settings
+version_remapping = {
+    "matplotlib": {"3.5.1": "3.5.0"},
+}
+
+
 def get_version(package_name: str) -> str:
     python_version = f"{sys.version_info.major}.{sys.version_info.minor}"
     constraints_path = f"../.constraints/py{python_version}.txt"
@@ -215,8 +224,16 @@ def get_version(package_name: str) -> str:
             continue
         if not line:
             continue
-        _, installed_version, *_ = tuple(line.split("=="))
+        line_segments = tuple(line.split("=="))
+        if len(line_segments) != 2:
+            continue
+        _, installed_version, *_ = line_segments
         installed_version = installed_version.strip()
+        remapped_versions = version_remapping.get(package_name)
+        if remapped_versions is not None:
+            existing_version = remapped_versions.get(installed_version)
+            if existing_version is not None:
+                return existing_version
         return installed_version
     return "stable"
 
@@ -236,11 +253,6 @@ intersphinx_mapping = {
         f"https://mpl-interactions.readthedocs.io/en/{get_version('mpl-interactions')}",
         None,
     ),
-    "numpy": ("https://numpy.org/doc/stable", None),
-    "pandas": (
-        f"https://pandas.pydata.org/pandas-docs/{get_version('pandas')}",
-        None,
-    ),
     "pwa": ("https://pwa.readthedocs.io", None),
     "python": ("https://docs.python.org/3", None),
     "qrules": (
